@@ -5,6 +5,7 @@ import pprint
 import sys
 import traceback
 
+from django.conf import settings
 from django.db.models import QuerySet, Model
 from django.forms import model_to_dict
 from websocket import create_connection
@@ -99,7 +100,9 @@ def JsonWSResponseWithException(printing=False, printing_res=False):
                 if not self.port:
                     self.port = request.session._session.get('ws_port')
                     if not self.port:
-                        raise Exception(Common.arraund_error('Do not have ws_port'))
+                        self.port = settings.WS_PORT
+                        if not self.port:
+                            raise Exception(Common.arraund_error('Do not have ws_port'))
 
             self.host = request.GET.get('ws_host')
             if not self.host:
@@ -107,7 +110,9 @@ def JsonWSResponseWithException(printing=False, printing_res=False):
                 if not self.host:
                     self.host = request.session._session.get('host')
                     if not self.host:
-                        raise Exception(Common.arraund_error('Do not have ws_host'))
+                        self.host = settings.WS_HOST
+                        if not self.host:
+                            raise Exception(Common.arraund_error('Do not have ws_host'))
 
             self.channel = request.GET.get('ws_channel')
             if not self.channel:
