@@ -1,14 +1,17 @@
 import logging
-import time
+from typing import List
 
 from bitfield import Bit, BitHandler
-from django.db import connection
+from django.core.exceptions import EmptyResultSet
+from django.db import connection, transaction
 from django.db.models import Manager, QuerySet, Q, ForeignKey
 from django.forms import model_to_dict
 
-from isc_common import getAttr, delAttr, setAttr
+from isc_common import delAttr, setAttr
+from isc_common.bit import TurnBitOn, TurnBitOff
 from isc_common.fields.related import ForeignKeyProtect
 from isc_common.http.DSRequest import DSRequest
+from isc_common.number import DelProps
 
 logger = logging.getLogger(__name__)
 
@@ -225,8 +228,8 @@ class CommonQuerySet(QuerySet):
             # raise Exception(f'Неизветный textMatchStyle: {textMatchStyle}')
 
     def getValue(self, criterion):
-        value = getAttr(criterion, 'value')
-        operator = getAttr(criterion, 'operator')
+        value = criterion.get('value')
+        operator = criterion.get('operator')
         if value == None:
             if operator == "isNull":
                 return True
@@ -252,30 +255,18 @@ class CommonQuerySet(QuerySet):
         res = Q()
         if isinstance(crireria, list):
             # <editor-fold desc="Fixed by Y.Andrew">
-            crireria = [item for item in crireria if item.get('fieldName') != 'ts']
-            len_list = len(crireria)
-            criteria = []
-            if len_list <= 1:
-                criteria = crireria
-            else:
-                for item in crireria:
-                    if (item.get('fieldName') == 'parent' and item.get('value') == None):
-                        ...
-                    elif (item.get('fieldName') == 'parent_id' and item.get('value') == None):
-                        ...
-                    else:
-                        criteria.append(item)
+            criteria = [item for item in crireria if item.get('fieldName') != 'ts']
             # </editor-fold>
 
             for criterion in criteria:
-                _criteria = getAttr(criterion, "criteria")
+                _criteria = criterion.get("criteria")
                 if _criteria:
                     if self.isNotOperator(operator):
-                        res.add(~Q(self.getCriteria(_criteria, getAttr(criterion, "operator"))), operator)
+                        res.add(~Q(self.getCriteria(_criteria, criterion.get("operator"))), operator)
                     else:
-                        res.add(Q(self.getCriteria(_criteria, getAttr(criterion, "operator"))), operator)
-                elif getAttr(criterion, 'fieldName') != 'ts':
-                    _operator = self.getOperator(getAttr(criterion, 'operator'))
+                        res.add(Q(self.getCriteria(_criteria, criterion.get("operator"))), operator)
+                elif criterion.get('fieldName') != 'ts':
+                    _operator = self.getOperator(criterion.get('operator'))
                     if _operator == "notnull":
                         _operator = "isnull"
 
@@ -284,19 +275,19 @@ class CommonQuerySet(QuerySet):
                     else:
                         _operator = f'__{_operator}'
 
-                    if self.isNotOperator(getAttr(criterion, 'operator')):
-                        res.add(~Q(**{f"{getAttr(criterion, 'fieldName')}{_operator}": self.getValue(criterion)}), operator)
+                    if self.isNotOperator(criterion.get('operator')):
+                        res.add(~Q(**{f"{criterion.get('fieldName')}{_operator}": self.getValue(criterion)}), operator)
                     else:
                         if _operator == '__icontains':
                             if self.getValue(criterion) != '':
-                                res.add(Q(**{f"{getAttr(criterion, 'fieldName')}{_operator}": self.getValue(criterion)}), operator)
+                                res.add(Q(**{f"{criterion.get('fieldName')}{_operator}": self.getValue(criterion)}), operator)
                         else:
-                            res.add(Q(**{f"{getAttr(criterion, 'fieldName')}{_operator}": self.getValue(criterion)}), operator)
+                            res.add(Q(**{f"{criterion.get('fieldName')}{_operator}": self.getValue(criterion)}), operator)
 
         return res
 
     def get_criteria(self, json=dict()):
-        data = getAttr(json, 'data', dict())
+        data = json.get('data', dict())
         _data = dict()
         if data != None:
             for key, value in data.items():
@@ -317,14 +308,13 @@ class CommonQuerySet(QuerySet):
 
         data = _data
 
-        # todo Удалить несуществующие поля
         criteria = Q()
         operator = Q.AND
 
-        if getAttr(data, '_constructor') == 'AdvancedCriteria':
-            operator = self.getOperator(getAttr(data, "operator"))
-            _criteria = getAttr(data, "criteria")
-            if self.isNotOperator(getAttr(data, "operator")):
+        if data.get('_constructor') == 'AdvancedCriteria':
+            operator = self.getOperator(data.get("operator"))
+            _criteria = data.get("criteria")
+            if self.isNotOperator(data.get("operator")):
                 criteria.add(~Q(self.getCriteria(_criteria, operator)), operator)
             else:
                 criteria.add(Q(self.getCriteria(_criteria, operator)), operator)
@@ -333,7 +323,7 @@ class CommonQuerySet(QuerySet):
             # "exactCase"	case-sensitive exact match ("foo" matches only "foo")                       exact
             # "substring"	case-insenstive substring match ("foo" matches "foobar" and "BarFoo")       icontains
             # "startsWith"	case-insensitive prefix match ("foo" matches "FooBar" but not "BarFoo")     istartswith
-            textMatchStyle = self.textMatchStyleMapping(getAttr(json, 'textMatchStyle'))
+            textMatchStyle = self.textMatchStyleMapping(json.get('textMatchStyle'))
             for key, value in data.items():
                 if isinstance(value, Bit):
                     criteria.add(Q(**{f"{key}": value}), operator)
@@ -359,9 +349,9 @@ class CommonQuerySet(QuerySet):
 
         criteria = self.get_criteria(json=json)
 
-        ob = getAttr(json, "sortBy", [])
+        ob = json.get("sortBy", [])
 
-        if isinstance(distinct_field_names, tuple):
+        if isinstance(distinct_field_names, tuple) or (isinstance(distinct_field_names, list)):
             if self.alive_only is False:
                 if start is not None and end is not None:
                     queryResult = super().filter(*args, criteria).distinct(*distinct_field_names)[start:end]
@@ -401,10 +391,14 @@ class CommonQuerySet(QuerySet):
                     queryResult = super().order_by(*ob).filter(*args, criteria).filter(deleted_at__isnull=True)
         return queryResult
 
-    def get_range_rows(self, start=None, end=None, function=None, json=None, distinct_field_names=None, *args):
+    def get_range_rows(self, start=None, end=None, function=None, json=None, distinct_field_names=None, *args, **kwargs):
         queryResult = self._get_range_rows(*args, start=start, end=end, function=function, json=json, distinct_field_names=distinct_field_names)
 
-        logger.debug(f'\n\n{queryResult.query}\n')
+        try:
+            logger.debug(f'\n\n{queryResult.query}\n')
+        except EmptyResultSet:
+            pass
+
         if function:
             res = [function(record) for record in queryResult]
             return res
@@ -412,11 +406,20 @@ class CommonQuerySet(QuerySet):
             res = [model_to_dict(record) for record in queryResult]
             return res
 
-    def get_range_rows1(self, request, function=None, distinct_field_names=None):
+    def get_range_rows1(self, request, function=None, distinct_field_names=None, remove_fields=None):
         request = DSRequest(request=request)
+        if isinstance(remove_fields, list):
+            data = request.json.get('data')
+            deleted = False
+            for remove_field in remove_fields:
+                if delAttr(data, remove_field) == True:
+                    deleted = True
+            if deleted == True:
+                setAttr(request.json, 'data', data)
+
         self.alive_only = request.alive_only
         self.enabledAll = request.enabledAll
-        res = self.get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json)
+        res = self.get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json, criteria=request.get_criteria())
         return res
 
     def get_range_rows2(self, *args, **kwargs):
@@ -491,6 +494,11 @@ class CommonManager(Manager):
         if fld and len(fld) > 0:
             return fld
 
+        if field_name.rfind('_id') + 3 == len(field_name):
+            fld = [x for x in model._meta.fields if x.name == field_name.replace('_id', '')]
+            if fld and len(fld) > 0:
+                return fld
+
         list_foreign = field_name.split('__')
         if len(list_foreign) > 1:
             list_foreign, spliter = [x.split('_') for x in list_foreign], '__'
@@ -520,16 +528,28 @@ class CommonManager(Manager):
 
     def get_range_rows1(self, request, function=None, distinct_field_names=None):
         request = DSRequest(request=request)
+        self.alive_only = request.alive_only
+        self.enabledAll = request.enabledAll
         return self.get_queryset().get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json)
 
-    def createFromRequest(self, request):
+    def createFromRequest(self, request, removed=None, propsArr: List = None):
         request = DSRequest(request=request)
         data = request.get_data()
         _data = data.copy()
+
+        if _data.get('parent') != None and isinstance(_data.get('parent'), int):
+            setAttr(_data, 'parent_id', _data.get('parent'))
+            delAttr(_data, 'parent')
+
         delAttr(_data, 'dataSource')
         delAttr(_data, 'operationType')
         delAttr(_data, 'textMatchStyle')
         delAttr(_data, 'form')
+        self._remove_prop(_data, removed)
+
+        props = self.get_prp(data=_data, propsArr=propsArr)
+        if props > -1:
+            setAttr(_data, 'props', props)
 
         res = super().create(**_data)
         try:
@@ -541,25 +561,21 @@ class CommonManager(Manager):
             setAttr(res, 'full_name', full_name)
 
         setAttr(res, 'isFolder', False)
-
-        props = getAttr(res, 'props')
-        if props and isinstance(props, BitHandler):
-            props = getAttr(res, 'props')._value
-            setAttr(res, 'props', props)
-        data.update(res)
+        data.update(DelProps(res))
         return data
 
     def _dataIsArray(self, data):
-        if not isinstance(data, dict):
-            return False
-
-        i = 0
-        for key in data.keys():
-            if key not in ['Class', 'localeStringFormatter']:
-                if key != str(i):
-                    return False
-                i += 1
-        return True
+        return isinstance(data, list)
+        # if not isinstance(data, dict):
+        #     return False
+        #
+        # i = 0
+        # for key in data.keys():
+        #     if key not in ['Class', 'localeStringFormatter']:
+        #         if key != str(i):
+        #             return False
+        #         i += 1
+        # return True
 
     def _remove_prop(self, data, removed):
         if isinstance(data, dict) and isinstance(removed, list):
@@ -572,7 +588,7 @@ class CommonManager(Manager):
             res = dict()
             for key in data.keys():
                 if not key.startswith('_'):
-                    setAttr(res, key, getAttr(data, key))
+                    setAttr(res, key, data.get(key))
         return res
 
     def clone_data(self, data):
@@ -583,7 +599,27 @@ class CommonManager(Manager):
                     setAttr(_data, key, val)
         return _data
 
-    def updateFromRequest(self, request, removed=None, function=None):
+    def get_prp(self, data, propsArr, props_name='props'):
+        if isinstance(data, dict):
+            props = data.get(props_name, 0)
+            if isinstance(propsArr, list):
+                idx = 0
+                for prop in propsArr:
+                    _prop = data.get(prop)
+
+                    if _prop == True:
+                        props = TurnBitOn(props, idx)
+                    else:
+                        props = TurnBitOff(props, idx)
+
+                    delAttr(data, prop)
+                    idx += 1
+                return props
+            else:
+                return -1
+        return -1
+
+    def updateFromRequest(self, request, removed=None, function=None, propsArr: List = None):
         if not isinstance(request, DSRequest):
             request = DSRequest(request=request)
         data = request.get_data()
@@ -599,21 +635,30 @@ class CommonManager(Manager):
                     if function:
                         function(data, data_item)
                     else:
+                        props = self.get_prp(data=data_item, propsArr=propsArr)
+                        cloned_data = self.clone_data(data)
+                        if props > -1:
+                            setAttr(cloned_data, 'props', props)
+
                         if data_item.get('id'):
-                            super().filter(id=data_item.get('id')).update(**self.clone_data(data_item))
+                            super().update_or_create(id=data_item.get('id'), defaults=cloned_data)
                         else:
-                            super().create(**self.clone_data(data))
+                            super().create(**cloned_data)
         else:
             self._remove_prop(data, removed)
             data = self._remove_prop_(data)
             if function:
                 function(data)
             else:
+                props = self.get_prp(data=data, propsArr=propsArr)
+                cloned_data = self.clone_data(data)
+                if props > -1:
+                    setAttr(cloned_data, 'props', props)
                 if request.get_id():
-                    cloned_data = self.clone_data(data)
-                    new_item = super().filter(id=request.get_id()).update(**cloned_data)
+                    delAttr(cloned_data, 'id')
+                    super().update_or_create(id=request.get_id(), defaults=cloned_data)
                 else:
-                    new_item = super().create(**self.clone_data(data))
+                    new_item = super().create(**cloned_data)
                     setAttr(_data, 'id', new_item.id)
         return _data
 
@@ -717,22 +762,26 @@ class CommonManager(Manager):
             field = field[0]
 
         if field:
-            if field.null and value == 'null':
+            if field == None:
                 setAttr(_data, 'id', None)
                 res = _data
             else:
-                res = field.related_model.objects.get(**_data)
-                res = model_to_dict(res)
-                for key, value in data.items():
-                    if str(key).find('full_name') != -1:
-                        setAttr(res, 'full_name', value)
+                res = field.related_model.objects.filter(**_data)
+                if len(res) >= 1:
+                    res = res[0]
+                    res = model_to_dict(res)
+                    for key, value in data.items():
+                        if str(key).find('full_name') != -1:
+                            setAttr(res, 'full_name', value)
+                else:
+                    logger.warning(f'Element {_data} not found')
 
         _res = {}
         if isinstance(res, dict):
             for key, value in res.items():
                 if not isinstance(value, list) and not isinstance(value, dict):
                     if isinstance(value, BitHandler):
-                        props = getAttr(res, 'props')._value
+                        props = res.get('props')._value
                         setAttr(_res, 'props', props)
                     else:
                         setAttr(_res, key, value)
@@ -742,12 +791,13 @@ class CommonManager(Manager):
         request = DSRequest(request=request)
         res = 0
         tuple_ids = request.get_tuple_ids()
-        for id, mode in tuple_ids:
-            if mode == 'hide':
-                super().filter(id=id).soft_delete()
-            else:
-                qty, _ = super().filter(id=id).delete()
-            res += qty
+        with transaction.atomic():
+            for id, mode in tuple_ids:
+                if mode == 'hide':
+                    super().filter(id=id).soft_delete()
+                else:
+                    qty, _ = super().filter(id=id).delete()
+                res += qty
         return res
 
     def get_ex(self, *args, **kwargs):
